#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Created by xiaojian on 2020/6/23
# Read xml file and parser for create table sql

__author__ = 'xiaojian'

import logging
import json
import xml.dom.minidom
from xml.dom import Node
from xml.dom.minidom import Text
from xml.dom.minidom import Element
from xml.dom.minidom import parseString

logging.basicConfig(level=logging.INFO)

xml_file = "asterix/config/asterix_cat062_1_18.xml"

dom = xml.dom.minidom.parse(xml_file)
root = dom.documentElement

VariableId = 1
resultJS = {}

def printJS(line):
    print(line)
    row = line.split("\t", -1)
    item_id = row[0]
    presence = row[1]
    presence_info = row[3]
    field = row[2]
    field_info = row[4]
    if len(presence) > 0:
        if len(field) > 0:
            resultJS[str.lower("_".join([item_id, presence, field]))] = field_info
        else:
            resultJS[str.lower("_".join([item_id, presence]))] = presence_info

    elif len(field) > 0:
        resultJS[str.lower("_".join([item_id, field]))] = field_info


def getBit(Bit, **kwargs):
    for text in Bit.childNodes:
        return text.nodeValue


def printBits(Bits, **kwargs):
    BitsShortName = ""
    BitsName = ""
    valIndex = 0
    for Bit in Bits.childNodes:
        if type(Bit) is not Element:
            continue
        if Bit.nodeName == "BitsShortName":
            BitsShortName = getBit(Bit, **kwargs)
        if Bit.nodeName == "BitsName":
            BitsName = getBit(Bit, **kwargs)
            if BitsName is None:
                BitsName = ""
        if Bit.nodeName == "BitsValue":
            BitsName += "{}:{};".format(valIndex,getBit(Bit, **kwargs))
            valIndex += 1
    if "Variable" in kwargs:
        global VariableId
        printJS("{}\t{}\t{}\t{}\t{}".format(kwargs["DataItemId"], kwargs["Variable"][str(VariableId)][0], BitsShortName,
                                            kwargs["Variable"][str(VariableId)][1], BitsName))
    else:
        printJS("{}\t{}\t{}\t{}\t{}".format(kwargs["DataItemId"], "", BitsShortName, "", BitsName))


def getBits(Bits, **kwargs):
    BitsPresence = ""
    BitsShortName = ""
    BitsName = ""
    for Bit in Bits.childNodes:
        if type(Bit) is not Element:
            continue
        if Bit.nodeName == "BitsPresence":
            BitsPresence = getBit(Bit, **kwargs)
        if Bit.nodeName == "BitsShortName":
            BitsShortName = getBit(Bit, **kwargs)
        if Bit.nodeName == "BitsName":
            BitsName = getBit(Bit, **kwargs)
    if BitsShortName == "spare":
        BitsPresence = "-1"
    if BitsShortName == "FX":
        BitsPresence = "0"
    return [BitsPresence, [BitsShortName, BitsName]]


def printFixed(Fixed, **kwargs):
    global VariableId
    for Bits in Fixed.childNodes:
        if type(Bits) is not Element:
            continue
        printBits(Bits, **kwargs)
    VariableId += 1


def gitFixed(Fixed, **kwargs):
    Variable = {}
    for Bits in Fixed.childNodes:
        if type(Bits) is not Element:
            continue
        [BitsPresence, [BitsShortName, BitsName]] = getBits(Bits, **kwargs)
        if BitsShortName is None:
            BitsShortName = ""
        if BitsName is None:
            BitsName = ""
        Variable[BitsPresence] = [BitsShortName, BitsName]
    return Variable


def getDataItemId(DataItem):
    if type(DataItem._attrs) == dict:
        return "I" + DataItem._attrs["id"]._value
    else:
        return -1


def printVariable(Variable, **kwargs):
    for Fixed in Variable.childNodes:
        if type(Fixed) is not Element:
            continue
        if Fixed.nodeName == "Fixed":
            printFixed(Fixed, DataItemId=DataItemId)


def getVariable(Variable, **kwargs):
    Variables = {}
    for Fixed in Variable.childNodes:
        if type(Fixed) is not Element:
            continue
        if Fixed.nodeName == "Fixed":
            Variable = gitFixed(Fixed, DataItemId=DataItemId)
            for key in Variable.keys():
                Variables[key] = Variable[key]
    return Variables


def printRepetitive(Repetitive, **kwargs):
    for DataItemFormat in Repetitive.childNodes:
        if type(DataItemFormat) is not Element:
            continue
        if DataItemFormat.nodeName == "Fixed":
            printFixed(DataItemFormat, **kwargs)


def printCompound(Compound, **kwargs):
    for DataItemFormat in Compound.childNodes:
        if type(DataItemFormat) is not Element:
            continue
        if DataItemFormat.nodeName == "Variable":
            kwargs["Variable"] = getVariable(DataItemFormat)
        else:
            if DataItemFormat.nodeName == "Fixed":
                printFixed(DataItemFormat, **kwargs)
            elif DataItemFormat.nodeName == "Repetitive":
                printRepetitive(DataItemFormat, **kwargs)
    for BitsPresence in ["-1", "0"]:
        if BitsPresence in kwargs["Variable"]:
            printJS("{}\t{}\t{}\t{}\t{}".format(kwargs["DataItemId"], kwargs["Variable"][BitsPresence][0], "",
                                                kwargs["Variable"][BitsPresence][1], ""))


def printExplicit(Explicit, **kwargs):
    for DataItemFormat in Explicit.childNodes:
        if type(DataItemFormat) is not Element:
            continue
        if DataItemFormat.nodeName == "Compound":
            printCompound(DataItemFormat, DataItemId=DataItemId)


def map2sql(map, key, depth):
    rows = []
    for k in map.keys():
        if type(map[k]) == str:
            rows.append(struct_str2sql(map, k, depth + 1))
        elif type(map[k]) == dict:
            rows.append(map2sql(map[k], k, depth + 1))
    if depth == 1:
        return """{}`_{}` struct \n{}<\n{}\n{}>""".format("\t\t" * depth, key, "\t\t" * depth, ",\n".join(rows),
                                                          "\t\t" * depth)
    else:
        return """{}`_{}`:struct \n{}<\n{}\n{}>""".format("\t\t" * depth, key, "\t\t" * depth, ",\n".join(rows),
                                                          "\t\t" * depth)


def str2sql(map, key, depth):
    return """ {}`{}` STRING comment "{}" """.format("\t\t" * depth, key, map[key])


def struct_str2sql(map, key, depth):
    return """ {}`{}`:STRING""".format("\t\t" * depth, key)


def json2sql(map):
    rows = []
    for key in map.keys():
        # if key != "I110":
        #     continue
        if type(map[key]) == str:
            rows.append(str2sql(map, key, 1))
        elif type(map[key]) == dict:
            rows.append(map2sql(map[key], key, 1))
    return rows


if __name__ == '__main__':

    for DataItem in root.childNodes:
        if type(DataItem) is not Element:
            continue

        DataItemId = getDataItemId(DataItem)
        for DataItemFormats in DataItem.childNodes:
            if type(DataItemFormats) is not Element:
                continue
            if DataItemFormats.nodeName == "DataItemFormat":
                for DataItemFormat in DataItemFormats.childNodes:
                    VariableId = 1
                    if type(DataItemFormat) is not Element:
                        continue
                    if DataItemFormat.nodeName == "Fixed":
                        printFixed(DataItemFormat, DataItemId=DataItemId)
                    if DataItemFormat.nodeName == "Variable":
                        printVariable(DataItemFormat, DataItemId=DataItemId)
                    if DataItemFormat.nodeName == "Compound":
                        printCompound(DataItemFormat, DataItemId=DataItemId)
                    if DataItemFormat.nodeName == "Explicit":
                        printExplicit(DataItemFormat, DataItemId=DataItemId)


    print("\n")
    print("======================================================================================")
    print("======                         json                                            =======")
    print("======================================================================================")

    print(json.dumps(resultJS,sort_keys=True, indent=4))
    print("\n")
    print("======================================================================================")
    print("======                    create table sql                                     =======")
    print("======================================================================================")
    sql = "CREATE TABLE IF NOT EXISTS TABLE_NAME (\n{}\n);".format(",\n".join(json2sql(resultJS)))
    print(sql)



#!/usr/bin/python

__author__ = 'dsalanti'

import socket
import struct
import asterix

sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM, socket.IPPROTO_UDP)
sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
sock.bind(('', 21111))
mreq = struct.pack("=4sl", socket.inet_aton("232.1.1.11"), socket.INADDR_ANY)
sock.setsockopt(socket.IPPROTO_IP, socket.IP_ADD_MEMBERSHIP, mreq)

counter = 1

while True:
    asterix_packet = sock.recv(10240)
    parsed = asterix.parse(asterix_packet)
    print('%d. Receiver received = %s' % (counter, parsed))
    counter += 1



#!/usr/bin/python

__author__ = 'dsalanti'

import threading
import time
import socket
import struct
import asterix

exitFlag = False

asterix_packet = bytes([0x30, 0x00, 0x30, 0xfd, 0xf7, 0x02, 0x19, 0xc9, 0x35, 0x6d, 0x4d, 0xa0, 0xc5, 0xaf, 0xf1, 0xe0,
                        0x02, 0x00, 0x05, 0x28, 0x3c, 0x66, 0x0c, 0x10, 0xc2, 0x36, 0xd4, 0x18, 0x20, 0x01, 0xc0, 0x78,
                        0x00, 0x31, 0xbc, 0x00, 0x00, 0x40, 0x0d, 0xeb, 0x07, 0xb9, 0x58, 0x2e, 0x41, 0x00, 0x20, 0xf5])


class SenderThread(threading.Thread):

    def __init__(self, threadID, name, counter):
        """
        Initialize thread
        :param threadID:
        :param name:
        :param counter:
        :return:
        """
        threading.Thread.__init__(self)
        self.threadID = threadID
        self.name = name
        self.counter = counter
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM, socket.IPPROTO_UDP)
        self.sock.setsockopt(socket.IPPROTO_IP, socket.IP_MULTICAST_TTL, 2)

    def run(self):
        """
        Send asterix packet
        :return:
        """
        print("Starting " + self.name)
        while self.counter:
            if exitFlag:
                self.name.exit()
            time.sleep(0.1)
            print("%d. Sender sending : %s" % (self.counter, str(asterix_packet)))
            self.sock.sendto(asterix_packet, ("224.51.105.104", 5000))
            self.counter -= 1
        print("Exiting " + self.name)


class ReceiverThread(threading.Thread):

    def __init__(self, threadID, name, counter):
        """
        Initialize receiver thread
        :param threadID:
        :param name:
        :param counter:
        :return:
        """
        threading.Thread.__init__(self)
        self.threadID = threadID
        self.name = name
        self.counter = counter
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM, socket.IPPROTO_UDP)
        self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)

    def run(self):
        """
        Receive asterix packets
        :return:
        """
        print("Starting " + self.name)

        self.sock.bind(('', 5000))
        mreq = struct.pack("=4sl", socket.inet_aton("224.51.105.104"), socket.INADDR_ANY)
        self.sock.setsockopt(socket.IPPROTO_IP, socket.IP_ADD_MEMBERSHIP, mreq)

        while self.counter:
            if exitFlag:
                self.name.exit()
            asterix_packet = self.sock.recv(10240)
            parsed = asterix.parse(asterix_packet)

            # print('%d. Receiver received = %s' % (self.counter, parsed))
            print(asterix.describe(parsed))
            self.counter -= 1

        print("Exiting " + self.name)


# Create new threads
repeat = 100000
sender_thread = SenderThread(1, "Sender", repeat)
receiver_thread = ReceiverThread(1, "Receiver", repeat)

# Start new Threads
sender_thread.start()
receiver_thread.start()

print("Exiting Main Thread")



__author__ = 'dsalanti'

import asterix
import struct
import datetime

# Read example file from packet resources
sample_filename = '/home/dsalanti/PycharmProjects/asterix/install/my_samples/REC_RECSDPS_MAIN_ACC_SYS_TK.ff'

final_header_format = ">hBBBBBB"
final_header_size = struct.calcsize(final_header_format)

final_padding_format = "BBBB"
final_padding_size = struct.calcsize(final_padding_format)
final_padding_value = (0xA5, 0xA5, 0xA5, 0xA5)

with open(sample_filename, "rb") as f:
    packetnr = 1
    while True:
        header_bytes = f.read(final_header_size)
        if len(header_bytes) != final_header_size:
            break

        (data_size, board_nr, line_nr, rec_day, time1, time2, time3) = struct.unpack(final_header_format, header_bytes)
        timems = (time1 * 65536 + time2 * 256 + time3) * 10
        packet_size = data_size - final_header_size - final_padding_size

        timestamp = datetime.timedelta(milliseconds=timems)

        print("%d.\ttime=%s\tSize=%d\tboard=%d\tline=%d\tday=%d)" % (
        packetnr, timestamp, packet_size, board_nr, line_nr, rec_day))
        packetnr += 1

        packet_bytes = f.read(packet_size)

        # Parse packet
        packets = asterix.parse(packet_bytes)
        # print(packets)
        for packet in packets:
            print(packet)

        padding_bytes = f.read(4)
        padding = struct.unpack("BBBB", padding_bytes)
        if padding != final_padding_value:
            print("Wrong padding!!")
            exit(1)

print("Finished")



"""
Asterix is utility used for reading and parsing of EUROCONTROL ASTERIX protocol data from stdin, file or network multicast stream.

ASTERIX stands for All Purpose STructured EUROCONTROL SuRveillance Information EXchange.
It is an ATM Surveillance Data Binary Messaging Format which allows transmission of harmonised information between any surveillance and automation system.
ASTERIX defines the structure of the data to be exchanged over a communication medium, from the encoding of every bit of information up to the organisation of the data within a block of data - without any loss of information during the whole process.
More about ASTERIX protocol you can find here: http://www.eurocontrol.int/services/asterix

Asterix application was developed by Croatia Control Ltd.

This program comes with ABSOLUTELY NO WARRANTY. This is free software, and you are welcome to redistribute it under certain conditions. See COPYING file for details.

Example:

    # Import the asterix module::
    import asterix

    # Prepare your asterix binary packet. E.g.::
    asterix_packet = bytearray([0x30, 0x00, 0x30, 0xfd, 0xf7, 0x02, 0x19, 0xc9, 0x35, 0x6d, 0x4d, 0xa0, 0xc5, 0xaf, 0xf1, 0xe0,
                        0x02, 0x00, 0x05, 0x28, 0x3c, 0x66, 0x0c, 0x10, 0xc2, 0x36, 0xd4, 0x18, 0x20, 0x01, 0xc0, 0x78,
                        0x00, 0x31, 0xbc, 0x00, 0x00, 0x40, 0x0d, 0xeb, 0x07, 0xb9, 0x58, 0x2e, 0x41, 0x00, 0x20, 0xf5])


    # Parse the packet with asterix module. It will return the list of Asterix records::
    parsed = asterix.parse(asterix_packet)
    print(parsed)

    # If you want to see textual presentation of asterix packet use describe::
	print(asterix.describe(parsed))

    # If you do not need description of items and meaning of bit values, you can disable the verbose mode, to just get values::
    parsed = asterix.parse(asterix_packet, verbose=false)
    print(parsed)

    # All Asterix categories are defined with XML configuration file.
    # You can get a list of configuration files ith following command::
    config = asterix.list_configuration_files()
    print(config)

    # And then get specific configuration file::
    configfile = asterix.get_configuration_file('cat048')
    with open(configfile, "rt") as f:
        config = f.read()
        print(config)

    # If you want to create your own Asterix definition file, create XML file using dtd definition::
    dtd = asterix.get_configuration_file('dtd')

    # And then initialize asterix with your configuration::
    asterix.init(path_to_your_config_file)

"""
import _asterix

try:
    from lxml import etree
    lxml_found = True
except ImportError:
    lxml_found = False

import os
import sys
from datetime import datetime
from .version import __version__


# def set_callback(callback):
#    return _asterix.set_callback(callback)


# def hello(world):
#    return _asterix.hello(world)


def init(filename):
    """ Initializes asterix with XML category definition
    Args:
        filename: Path to XML definition file
    Returns:
        0: if OK
    Raises:
        SyntaxError: if there is a syntax error in XMl file
        RuntimeError: if there is some internal problem
        IOError: if XML file is not found or can't be read
        ValueError: if parameter is not string
        TypeError: if there is more or less than 1 parameter
    Example:
        >>> asterix.init(path_to_your_config_file)
    """
    return _asterix.init(filename)


def describeXML(category, item=None, field=None, value=None):
    """ Return the description of specific Category, Item, Field
    and Value in Asterix specification as a lxml ElementTree
    Args:
        category: Asterix category (e.g. 62)
        item: Asterix Item ID (e.g. 'I010')
        field: Asterix field name (e.g. 'SAC')
        value: Asterix field value meaning (e.g. 1)

    """
    if value:
        return _asterix.describeXML(category, item, field, value)
    elif field:
        return _asterix.describeXML(category, item, field)
    elif item:
        return _asterix.describeXML(category, item)
    return _asterix.describeXML(category)

def describe(category, item=None, field=None, value=None):
    """ Return the description of specific Category, Item, Field and Value in Asterix specification
    Args:
        category: Asterix category (e.g. 62)
        item: Asterix Item ID (e.g. 'I010')
        field: Asterix field name (e.g. 'SAC')
        value: Asterix field value meaning (e.g. 1)

    """
    if value:
        return _asterix.describe(category, item, field, value)
    elif field:
        return _asterix.describe(category, item, field)
    elif item:
        return _asterix.describe(category, item)
    return _asterix.describe(category)


def parse(data, **kwargs):
    """ Parse raw asterix data
    Args:
        data: Bytes to be parsed
    Kwargs:
        verbose=True: Show description, meaning, max and min values of item (default: True)
    Returns:
        list of asterix records
    """
    if 'verbose' in kwargs and not kwargs['verbose']:
        verbose = 0
    else:
        verbose = 1

    if sys.version_info <= (2, 7):
        return _asterix.parse(buffer(data), verbose)
    return _asterix.parse(bytes(data), verbose)


def parse_with_offset(data, offset=0, blocks_count=1000, **kwargs):
    """ Parse raw asterix data with bytes offset with returning number of blocks of data 
    passed with arguments
    Args:
        data: Bytes to be parsed
        offset: bytes offset
        blocks_count: number of blocks data to be returned
    Kwargs:
        verbose=True: Show description, meaning, max and min values of item (default: True)
    Returns:
        tuple of two elements:
            list of asterix records
            bytes offset at ending of computation
    """
    if 'verbose' in kwargs and not kwargs['verbose']:
        verbose = 0
    else:
        verbose = 1

    if sys.version_info <= (2, 7):
        return _asterix.parse_with_offset(buffer(data), offset, blocks_count, verbose)
    return _asterix.parse_with_offset(bytes(data), offset, blocks_count, verbose)


def describeXML(parsed, descriptions=False):
    """ Describe all elements in Asterix data in a lxml ElementTree.
    Content is identical to the command "./asterix --xml filename".
    Args:
        parsed: Parsed Asterix packet returned by asterix.parse
    Returns:
         lxml ElementTree describing all Asterix data
         None if lxml module is not installed
    """
    if lxml_found:
        xml = etree.Element('ASTERIXSTART')
        for record in parsed:
            xml_Record = etree.SubElement(xml, 'ASTERIX')
            cat = record['category']
            ts = record['ts']
            xml_Record.set('ver', '%d' % 1)
            xml_Record.set('length', '%ld' % record['len'])
            xml_Record.set('crc', '%s' % record['crc'])
            xml_Record.set('timestamp', '%ld' % ts)
            if 'hexdata' in record:
                xml_Record.set('hexdata', record['hexdata'])
            xml_Record.set('cat', '%03d' % cat)
            if descriptions:
                xml_Record.set('description', '%s' % _asterix.describe(cat))

            for key, value in record.items():
                if key[0] == 'I':
                    xml_Item = etree.SubElement(xml_Record, key)
                    description = '%s' % _asterix.describe(cat, str(key))
                    if description and descriptions:
                        xml_Item.set('description', description)
                    if isinstance(value, dict):
                        for ikey, ival in value.items():
                            xml_Field = etree.SubElement(xml_Item, ikey)
                            if 'val' in ival:
                                description1 = _asterix.describe(cat, str(key), ikey)
                                description2 = _asterix.describe(cat, str(key), ikey,
                                                                 str(ival['val']))
                                if description2 and descriptions:
                                    xml_Field.set('description', description2)
                                xml_Field.text = str(ival['val'])
                            else:
                                for ikey2, ival2 in ival.items():
                                    xml_subItem = etree.SubElement(xml_Field, ikey2)
                                    description1 = _asterix.describe(cat, str(key), ikey2)
                                    description2 = _asterix.describe(cat, str(key), ikey2,
                                                                     str(ival2['val']))
                                    if description2 and descriptions:
                                        xml_subItem.set('description', description2)
                                    xml_subItem.text = str(ival2['val'])
                    elif isinstance(value, list):
                        for it in value:
                            for ikey, ival in it.items():
                                xml_Field = etree.SubElement(xml_Item, ikey)
                                description1 = _asterix.describe(cat, str(key), ikey)
                                description2 = _asterix.describe(cat, str(key), ikey,
                                                                 str(ival['val']))
                                if description2 and descriptions:
                                    xml_Field.set('description', description2)
                                xml_Field.text = str(ival['val'])
                    else:
                        xml_Item.text = str(value)

        return xml
    return None

def describe(parsed):
    """ Describe all elements in Asterix data in textual format.
    Args:
        parsed: Parsed Asterix packet returned by ateris.parse
    Returns:
         Formatted string describing all Asterix data
    """
    i = 0
    txt = ''
    for record in parsed:
        i += 1
        txt += '\n\nAsterix record: %d ' % i
        len = record['len']
        txt += '\nLen: %ld' % (len)
        crc = record['crc']
        txt += '\nCRC: %s' % (crc)
        ts = record['ts']
        strts = datetime.fromtimestamp(ts / 1000.)

        txt += '\nTimestamp: %ld (%s)' % (ts, strts)
        cat = record['category']
        txt += '\nCategory: %d (%s)' % (cat, _asterix.describe(cat))
        txt += '\nHexdata: %s' % record['hexdata']

        for key, value in record.items():
            if key != 'category':
                if isinstance(value, dict):
                    for ikey, ival in value.items():
                        txt += '\nItem: %s (%s)' % (str(key), _asterix.describe(cat, str(key)))

                        if 'val' in ival:
                            txt += '\n\t%s (%s): %s %s' % (
                            ikey, _asterix.describe(cat, str(key), ikey), str(ival['val']),
                            _asterix.describe(cat, str(key), ikey, str(ival['val'])))
                        else:
                            txt += '\n\t%s' % ikey
                            for ikey2, ival2 in ival.items():
                                txt += '\n\t\t%s (%s): %s %s' % (
                                ikey2, _asterix.describe(cat, str(key), ikey2), str(ival2['val']),
                                _asterix.describe(cat, str(key), ikey2, str(ival2['val'])))
                elif isinstance(value, list):
                    txt += '\nItem: %s (%s)' % (str(key), _asterix.describe(cat, str(key)))

                    for it in value:
                        for ikey, ival in it.items():
                            txt += '\n\t%s (%s): %s %s' % (
                            ikey, _asterix.describe(cat, str(key), ikey), str(ival['val']),
                            _asterix.describe(cat, str(key), ikey, str(ival['val'])))

    return txt


def list_sample_files():
    """ Return the list of Asterix format sample files from the package
    Returns:
         list of Asterix sample files
    Example:
        >>> list_sample_files()
    """
    sample_files = []
    filepath = os.path.join(os.path.dirname(__file__), 'sample_data')
    for fn in sorted(os.listdir(filepath)):
        f = os.path.join(filepath, fn)
        if os.path.isfile(f):
            sample_files.append(f)
    return sample_files


def get_sample_file(match):
    """ Returns first Asterix sample file matching the parameter string
    Args:
        match: Search string for sample file
    Returns:
         Sample file path
    """
    filepath = os.path.join(os.path.dirname(__file__), 'sample_data')
    for fn in sorted(os.listdir(filepath)):
        f = os.path.join(filepath, fn)
        if os.path.isfile(f) and match in fn:
            return f


def list_configuration_files():
    """ Return the list of Asterix configuration files from the package
    Returns:
        list of Asterix configuration files
    """
    sample_files = []
    filepath = os.path.join(os.path.dirname(__file__), 'config')
    for fn in sorted(os.listdir(filepath)):
        f = os.path.join(filepath, fn)
        if os.path.isfile(f):
            sample_files.append(f)
    return sample_files


def get_configuration_file(match):
    """ Returns first Asterix configuration file matching the parameter string
    Args:
        match: Search string for configuration file
    Returns:
         Configuration file path
    """
    filepath = os.path.join(os.path.dirname(__file__), 'config')
    for fn in sorted(os.listdir(filepath)):
        f = os.path.join(filepath, fn)
        if os.path.isfile(f) and match in fn:
            return f


# default callback function
# def callback(arg):
#    for a in arg:
#        print(a)

# initialize asterix with default configuration files

filepath = os.path.join(os.path.dirname(__file__), 'config')
for fn in sorted(os.listdir(filepath)):
    f = os.path.join(filepath, fn)
    if os.path.isfile(f) and f.endswith(".xml"):
        _asterix.init(f)

# set default callback
# _asterix.set_callback(callback)



__author__ = 'dsalanti'

import asterix

# This is binary presentation of asterix packet of CAT048
#                  len  cat 048
#                 ---- ---------
asterix_packet = bytearray(
    [0x30, 0x00, 0x30, 0xfd, 0xf7, 0x02, 0x19, 0xc9, 0x35, 0x6d, 0x4d, 0xa0, 0xc5, 0xaf, 0xf1, 0xe0,
     0x02, 0x00, 0x05, 0x28, 0x3c, 0x66, 0x0c, 0x10, 0xc2, 0x36, 0xd4, 0x18, 0x20, 0x01, 0xc0, 0x78,
     0x00, 0x31, 0xbc, 0x00, 0x00, 0x40, 0x0d, 0xeb, 0x07, 0xb9, 0x58, 0x2e, 0x41, 0x00, 0x20, 0xf5])

# parse and print packet
parsed = asterix.parse(asterix_packet)

# describe Asterix data
formatted = asterix.describe(parsed)
print(formatted)



__author__ = 'dsalanti'

import asterix
import dpkt

# Read example file from packet resources
sample_filename = asterix.get_sample_file('cat_034_048.pcap')
with open(sample_filename, 'rb') as f:
    pcap = dpkt.pcap.Reader(f)

    cntr = 1
    for ts, buf in pcap:
        eth = dpkt.ethernet.Ethernet(buf)
        data = eth.ip.udp.data

        hexdata = ":".join("{:02x}".format(ord(c)) for c in str(data))
        print('Parsing packet %d : %s' % (cntr, hexdata))
        cntr += 1

        # Parse data
        parsed = asterix.parse(data)
        print(parsed)



__author__ = 'dsalanti'

import asterix

print(asterix.__version__)

# Read example file from packet resources
sample_filename = asterix.get_sample_file('cat062cat065.raw')

with open(sample_filename, "rb") as f:
    data = f.read()

    # Parse data description=True
    print('Items with description')
    print('----------------------')
    parsed = asterix.parse(data)
    for packet in parsed:
        for item in packet.items():
            print(item)

    print('Items without description')
    print('----------------------')
    # Parse data description=False
    parsed = asterix.parse(data, verbose=False)
    for packet in parsed:
        for item in packet.items():
            print(item)

    print('Textual description of data')
    print('----------------------')
    # describe Asterix data
    formatted = asterix.describe(parsed)
    print(formatted)



import asterix
import unittest


class AsterixParseTest(unittest.TestCase):
    def test_ParseCAT048(self):
        sample_filename = asterix.get_sample_file('cat048.raw')
        with open(sample_filename, "rb") as f:
            data = f.read()
            packet = asterix.parse(data)
            self.maxDiff = None
            self.assertIsNotNone(packet)
            self.assertIsNotNone(packet[0])
            self.assertIs(len(packet), 1)
            self.assertTrue('I220' in packet[0])
            self.assertEqual(packet[0]['category'], 48)
            self.assertEqual(packet[0]['len'], 45)
            self.assertEqual(packet[0]['crc'], 'AB659C3E')
            self.assertTrue('ts' in packet[0])
            self.assertEqual(packet[0]['I220']['ACAddr']['val'], '3C660C')
            self.assertEqual(packet[0]['I220']['ACAddr']['desc'], 'AircraftAddress')
            self.assertEqual(packet[0]['I010'], {'SAC': {'desc': 'System Area Code', 'val': 25},
                                                 'SIC': {'desc': 'System Identification Code', 'val': 201}})
            self.assertEqual(packet[0]['I170'], {'GHO': {'desc': 'GHO', 'val': 0, 'meaning': 'True target track'},
                                                 'TCC': {'desc': 'TCC', 'val': 0, 'meaning': 'Radar plane'},
                                                 'RAD': {'desc': 'RAD', 'val': 2, 'meaning': 'SSR/ModeS Track'},
                                                 'spare': {'desc': 'spare bits set to 0', 'const': 0, 'val': 0},
                                                 'TRE': {'desc': 'TRE', 'val': 0, 'meaning': 'Track still alive'},
                                                 'CDM': {'desc': 'CDM', 'val': 0, 'meaning': 'Maintaining'},
                                                 'CNF': {'desc': 'CNF', 'val': 0, 'meaning': 'Confirmed Track'},
                                                 'SUP': {'desc': 'SUP', 'val': 0,
                                                         'meaning': 'Track from cluster network - NO'},
                                                 'FX': {'desc': 'FX', 'val': 0, 'meaning': 'End of Data Item'},
                                                 'DOU': {'desc': 'DOU', 'val': 0, 'meaning': 'Normal confidence'},
                                                 'MAH': {'desc': 'MAH', 'val': 0,
                                                         'meaning': 'No horizontal man. sensed'}})
            self.assertEqual(packet[0]['I200'], {'CGS': {'desc': 'Calculated groundspeed', 'val': 434.3994140625},
                                                 'CHdg': {'desc': 'Calculated heading', 'val': 124.002685546875}})
            self.assertEqual(packet[0]['I220'], {'ACAddr': {'desc': 'AircraftAddress', 'val': '3C660C'}})

            self.assertEqual(packet[0]['I250'][0], {'TARGET_ALT_STATUS': {'desc': 'Status of Target ALT source bits',
                                                                          'meaning': 'No source information provided',
                                                                          'val': 0},
                                                    'res1': {'desc': 'Reserved 1', 'val': 0},
                                                    'res2': {'desc': 'Reserved 2', 'val': 0},
                                                    'FMS_ALT': {'desc': 'FMS Selected Altitude', 'val': 0.0},
                                                    'APP': {'desc': 'APPROACH Mode', 'meaning': 'Not active', 'val': 0},
                                                    'ALT_HOLD': {'desc': 'ALT HOLD Mode', 'meaning': 'Not active',
                                                                 'val': 0},
                                                    'TARGET_ALT_SOURCE': {'desc': 'Target ALT source',
                                                                          'meaning': 'Unknown', 'val': 0},
                                                    'BDS': {'desc': 'BDS register', 'val': '40'},
                                                    'FMS_ALT_STATUS': {'desc': 'FMS Altitude Status', 'val': 0},
                                                    'BP_STATUS': {'desc': 'Barometric Pressure Status', 'val': 1},
                                                    'BP': {'desc': 'Barometric Pressure', 'val': 227.0},
                                                    'MODE_STATUS': {'desc': 'Status of MCP/FCU Mode Bits', 'val': 0},
                                                    'VNAV': {'desc': 'VNAV Mode', 'meaning': 'Not active', 'val': 0},
                                                    'MCP_ALT_STATUS': {'desc': 'MCP Altitude Status', 'val': 1},
                                                    'MCP_ALT': {'desc': 'MCP/FCU Selected Altitude', 'val': 33008.0}})

            self.assertEqual(packet[0]['I040'], {'THETA': {'desc': '', 'val': 340.13671875},
                                                 'RHO': {'desc': '', 'max': 256.0, 'val': 197.68359375}})
            self.assertEqual(packet[0]['I240'],
                             {'TId': {'desc': 'Characters 1-8 (coded on 6 bits each) defining target identification',
                                      'val': 'DLH65A  '}})
            self.assertEqual(packet[0]['I140'], {'ToD': {'desc': 'Time Of Day', 'val': 27354.6015625}})
            self.assertEqual(packet[0]['I070'], {'Mode3A': {'desc': 'Mode-3/A reply code', 'val': '1000'},
                                                 'V': {'desc': '', 'val': 0, 'meaning': 'Code validated'},
                                                 'L': {'desc': '', 'val': 0,
                                                       'meaning': 'Mode-3/A code as derived from the reply of the transponder'},
                                                 'spare': {'desc': 'spare bit set to 0', 'const': 0, 'val': 0},
                                                 'G': {'desc': '', 'val': 0, 'meaning': 'Default'}})
            self.assertEqual(packet[0]['I161'], {'Tn': {'desc': 'Track Number', 'val': 3563}})
            self.assertEqual(packet[0]['I020'], {'SIM': {'desc': 'SIM', 'val': 0, 'meaning': 'Actual target report'},
                                                 'TYP': {'desc': 'TYP', 'val': 5, 'meaning': 'Single ModeS Roll-Call'},
                                                 'RAB': {'desc': 'RAB', 'val': 0,
                                                         'meaning': 'Report from aircraft transponder'},
                                                 'RDP': {'desc': 'RDP', 'val': 0,
                                                         'meaning': 'Report from RDP Chain 1'},
                                                 'FX': {'desc': 'FX', 'val': 0, 'meaning': 'End of Data Item'},
                                                 'SPI': {'desc': 'SPI', 'val': 0, 'meaning': 'Absence of SPI'}})
            self.assertEqual(packet[0]['I090'], {'V': {'desc': '', 'val': 0, 'meaning': 'Code validated'},
                                                 'FL': {'desc': 'FlightLevel', 'val': 330.0},
                                                 'G': {'desc': '', 'val': 0, 'meaning': 'Default'}})
            self.assertEqual(packet[0]['I230'], {'COM': {'desc': 'COM', 'val': 1},
                                                 'BDS37': {'desc': 'BDS 1,0 bits 37/40', 'val': 5},
                                                 'ModeSSSC': {'desc': 'ModeS Specific Service Capability', 'val': 1,
                                                              'meaning': 'Yes'}, 'STAT': {'desc': 'STAT', 'val': 0},
                                                 'AIC': {'desc': 'Aircraft identification capability', 'val': 1,
                                                         'meaning': 'Yes'},
                                                 'BDS16': {'desc': 'BDS 1,0 bit 16', 'val': 1},
                                                 'spare': {'desc': 'spare bit set to 0', 'const': 0, 'val': 0},
                                                 'ARC': {'desc': 'Altitude reporting capability', 'val': 1,
                                                         'meaning': '25ft resolution'},
                                                 'SI': {'desc': 'SI/II Transponder Capability', 'val': 0,
                                                        'meaning': 'SI-Code Capable'}})

    def test_ParseCAT048_nodescription(self):
        sample_filename = asterix.get_sample_file('cat048.raw')
        with open(sample_filename, "rb") as f:
            data = f.read()
            packet = asterix.parse(data, verbose=False)
            self.maxDiff = None
            self.assertIsNotNone(packet)
            self.assertIsNotNone(packet[0])
            self.assertIs(len(packet), 1)
            self.assertTrue('I220' in packet[0])
            self.assertEqual(packet[0]['category'], 48)
            self.assertEqual(packet[0]['len'], 45)
            self.assertEqual(packet[0]['crc'], 'AB659C3E')
            self.assertTrue('ts' in packet[0])
            self.assertEqual(packet[0]['I220']['ACAddr']['val'], '3C660C')
            self.assertEqual(packet[0]['I010'], {'SAC': {'val': 25},
                                                 'SIC': {'val': 201}})
            self.assertEqual(packet[0]['I170'], {'GHO': {'val': 0},
                                                 'TCC': {'val': 0},
                                                 'RAD': {'val': 2},
                                                 'spare': {'val': 0},
                                                 'TRE': {'val': 0},
                                                 'CDM': {'val': 0},
                                                 'CNF': {'val': 0},
                                                 'SUP': {'val': 0},
                                                 'FX': {'val': 0},
                                                 'DOU': {'val': 0},
                                                 'MAH': {'val': 0}})
            self.assertEqual(packet[0]['I200'], {'CGS': {'val': 434.3994140625},
                                                 'CHdg': {'val': 124.002685546875}})
            self.assertEqual(packet[0]['I220'], {'ACAddr': {'val': '3C660C'}})
            self.assertEqual(packet[0]['I250'][0], {'TARGET_ALT_STATUS': {'val': 0},
                                                    'res1': {'val': 0},
                                                    'res2': {'val': 0},
                                                    'FMS_ALT': {'val': 0.0},
                                                    'APP': {'val': 0},
                                                    'ALT_HOLD': {'val': 0},
                                                    'TARGET_ALT_SOURCE': {'val': 0},
                                                    'BDS': {'val': '40'},
                                                    'FMS_ALT_STATUS': {'val': 0},
                                                    'BP_STATUS': {'val': 1},
                                                    'BP': {'val': 227.0},
                                                    'MODE_STATUS': {'val': 0},
                                                    'VNAV': {'val': 0},
                                                    'MCP_ALT_STATUS': {'val': 1},
                                                    'MCP_ALT': {'val': 33008.0}})
            self.assertEqual(packet[0]['I040'], {'THETA': {'val': 340.13671875},
                                                 'RHO': {'val': 197.68359375}})
            self.assertEqual(packet[0]['I240'], {'TId': {'val': 'DLH65A  '}})
            self.assertEqual(packet[0]['I140'], {'ToD': {'val': 27354.6015625}})
            self.assertEqual(packet[0]['I070'], {'Mode3A': {'val': '1000'},
                                                 'V': {'val': 0},
                                                 'L': {'val': 0},
                                                 'spare': {'val': 0},
                                                 'G': {'val': 0}})
            self.assertEqual(packet[0]['I161'], {'Tn': {'val': 3563}})
            self.assertEqual(packet[0]['I020'], {'SIM': {'val': 0},
                                                 'TYP': {'val': 5},
                                                 'RAB': {'val': 0},
                                                 'RDP': {'val': 0},
                                                 'FX': {'val': 0},
                                                 'SPI': {'val': 0}})
            self.assertEqual(packet[0]['I090'], {'V': {'val': 0},
                                                 'FL': {'val': 330.0},
                                                 'G': {'val': 0}})
            self.assertEqual(packet[0]['I230'], {'COM': {'val': 1},
                                                 'BDS37': {'val': 5},
                                                 'ModeSSSC': {'val': 1},
                                                 'STAT': {'val': 0},
                                                 'AIC': {'val': 1},
                                                 'BDS16': {'val': 1},
                                                 'spare': {'val': 0},
                                                 'ARC': {'val': 1},
                                                 'SI': {'val': 0}})

    def test_ParseCAT062CAT065(self):
        self.maxDiff = None
        sample_filename = asterix.get_sample_file('cat062cat065.raw')
        with open(sample_filename, "rb") as f:
            data = f.read()
            packet = asterix.parse(data)
            self.assertIsNotNone(packet)
            self.assertIsNotNone(packet[0])
            self.assertIs(len(data), 195)
            self.assertIs(len(packet), 3)
            self.assertIs(packet[0]['category'], 62)
            self.assertIs(packet[0]['len'], 66)
            self.assertEqual(packet[0]['crc'], '943598B9')
            self.assertIs(packet[1]['category'], 62)
            self.assertIs(packet[1]['len'], 114)
            self.assertEqual(packet[1]['crc'], '0F55FD0E')
            self.assertIs(packet[2]['category'], 65)
            self.assertIs(packet[2]['len'], 9)
            self.assertEqual(packet[2]['crc'], 'ED9D3EB1')
            self.assertEqual(packet[0]['I220'], {'RoC': {'val': -443.75, 'desc': 'Rate of Climb/Descent'}})
            self.assertEqual(packet[0]['I015'], {'SID': {'val': 4, 'desc': 'Service Identification'}})

            self.assertEqual(packet[0]['I290']['MDS'], {
                'MDS': {'val': 63.75, 'desc': 'Age of the last Mode S detection used to update the track'}})
            self.assertEqual(packet[0]['I290']['PSR'], {
                'PSR': {'val': 7.25, 'desc': 'Age of the last primary detection used to update the track'}})
            self.assertEqual(packet[0]['I290']['SSR'], {
                'SSR': {'val': 0.0, 'desc': 'Age of the last secondary detection used to update the track'}})

            self.assertEqual(packet[0]['I135'], {
                'QNH': {'meaning': 'No QNH correction applied', 'val': 0, 'desc': 'QNH'},
                'CTBA': {'max': 150000.0, 'min': -1500.0, 'val': 15700.0,
                         'desc': 'Calculated Track Barometric Alt'}})
            self.assertEqual(packet[0]['I136'], {
                'MFL': {'max': 150000.0, 'min': -1500.0, 'val': 15700.0, 'desc': 'Measured Flight Level'}})
            self.assertEqual(packet[0]['I185'], {'Vx': {'max': 8191.75, 'min': -8192.0, 'val': -51.25, 'desc': 'Vx'},
                                                 'Vy': {'max': 8191.75, 'min': -8192.0, 'val': 170.0, 'desc': 'Vy'}})
            self.assertEqual(packet[0]['I080'], {'STP': {'meaning': 'default value', 'val': 0, 'desc': ''},
                                                 'MD5': {'meaning': 'No Mode 5 interrogation', 'val': 0, 'desc': ''},
                                                 'FPC': {'meaning': 'Not flight-plan correlated', 'val': 0, 'desc': ''},
                                                 'AMA': {'meaning': 'track not resulting from amalgamation process',
                                                         'val': 0, 'desc': ''},
                                                 'CNF': {'meaning': 'Confirmed track', 'val': 0, 'desc': ''},
                                                 'TSE': {'meaning': 'default value', 'val': 0, 'desc': ''},
                                                 'ME': {'meaning': 'default value', 'val': 0, 'desc': ''},
                                                 'FX': {'meaning': 'End of data item', 'val': 0, 'desc': ''},
                                                 'CST': {'meaning': 'Default value', 'val': 0, 'desc': ''},
                                                 'PSR': {'meaning': 'Default value', 'val': 0, 'desc': ''}, 'MDS': {
                    'meaning': 'Age of the last received Mode S track update is higher than system dependent threshold',
                    'val': 1, 'desc': ''},
                                                 'MI': {'meaning': 'default value', 'val': 0, 'desc': ''},
                                                 'SRC': {'meaning': 'height from coverage', 'val': 4,
                                                         'desc': 'Source of calculated track altitude for I062/130'},
                                                 'SIM': {'meaning': 'Actual track', 'val': 0, 'desc': ''},
                                                 'KOS': {'meaning': 'Background service used', 'val': 1, 'desc': ''},
                                                 'AFF': {'meaning': 'default value', 'val': 0, 'desc': ''},
                                                 'MRH': {'meaning': 'Barometric altitude (Mode C) more reliable',
                                                         'val': 0, 'desc': 'Most Reliable Height'},
                                                 'MON': {'meaning': 'Multisensor track', 'val': 0, 'desc': ''},
                                                 'TSB': {'meaning': 'default value', 'val': 0, 'desc': ''},
                                                 'SUC': {'meaning': 'Default value', 'val': 0, 'desc': ''},
                                                 'MD4': {'meaning': 'No Mode 4 interrogation', 'val': 0, 'desc': ''},
                                                 'SPI': {'meaning': 'default value', 'val': 0, 'desc': ''}, 'ADS': {
                    'meaning': 'Age of the last received ADS-B track update is higher than system dependent threshold',
                    'val': 1, 'desc': ''},
                                                 'AAC': {'meaning': 'Default value', 'val': 0, 'desc': ''},
                                                 'SSR': {'meaning': 'Default value', 'val': 0, 'desc': ''}})
            self.assertEqual(packet[0]['I070'], {'ToT': {'val': 30911.6640625, 'desc': 'Time Of Track Information'}})
            self.assertEqual(packet[0]['I100'], {'Y': {'val': -106114.0, 'desc': 'Y'},
                                                 'X': {'val': -239083.0, 'desc': 'X'}})
            self.assertEqual(packet[0]['I200'], {'VERTA': {'meaning': 'Descent', 'val': 2, 'desc': 'Vertical Rate'},
                                                 'spare': {'const': 0, 'val': 0, 'desc': 'Spare bit set to zero'},
                                                 'LONGA': {'meaning': 'Decreasing Groundspeed', 'val': 2,
                                                           'desc': 'Longitudinal Acceleration'},
                                                 'TRANSA': {'meaning': 'Constant Course', 'val': 0,
                                                            'desc': 'Transversal Acceleration'},
                                                 'ADF': {'meaning': 'No altitude discrepancy', 'val': 0,
                                                         'desc': 'Altitude Discrepancy Flag'}})
            self.assertEqual(packet[0]['I130'], {
                'Alt': {'max': 150000.0, 'min': -1500.0, 'val': 43300.0, 'desc': 'Altitude'}})
            self.assertEqual(packet[0]['I060'], {'CH': {'meaning': 'No Change', 'val': 0, 'desc': 'Change in Mode 3/A'},
                                                 'G': {'desc': 'Change in Mode 3/A', 'meaning': 'Default', 'val': 0},
                                                 'V': {'desc': 'Change in Mode 3/A', 'meaning': 'Code validated',
                                                       'val': 0},
                                                 'spare': {'const': 0, 'val': 0, 'desc': 'Spare bits set to 0'},
                                                 'Mode3A': {'val': '4276',
                                                            'desc': 'Mode-3/A reply in octal representation'}})

            self.assertEqual(packet[0]['I295']['MDA'], {'MDA': {'val': 0, 'desc': ''}})
            self.assertEqual(packet[0]['I295']['MFL'], {'MFL': {'val': 0.0, 'desc': ''}})

            self.assertEqual(packet[0]['I010'], {'SAC': {'val': 25, 'desc': 'System Area Code'},
                                                 'SIC': {'val': 100, 'desc': 'System Identification Code'}})

            self.assertEqual(packet[0]['I340']['TYP'], {
                'TYP': {'val': 2, 'meaning': 'Single SSR detection', 'desc': 'Report Type'},
                'TST': {'val': 0, 'meaning': 'Real target report', 'desc': ''},
                'spare': {'val': 0, 'desc': 'Spare bits set to zero', 'const': 0},
                'RAB': {'val': 0, 'meaning': 'Report from aircraft transponder', 'desc': ''},
                'SIM': {'val': 0, 'meaning': 'Actual target report', 'desc': ''}})

            self.assertEqual(packet[0]['I340']['SID'], {
                'SAC': {'val': 25, 'desc': 'System Area Code'},
                'SIC': {'val': 13, 'desc': 'System Identification Code'}})

            self.assertEqual(packet[0]['I340']['MDC'], {
                'CG': {'val': 0, 'meaning': 'Default', 'desc': ''},
                'CV': {'val': 0, 'meaning': 'Code validated', 'desc': ''},
                'ModeC': {'max': 1270.0, 'val': 157.0, 'min': -12.0, 'desc': 'Last Measured Mode C Code'}})

            self.assertEqual(packet[0]['I340']['MDA'], {
                'L': {'val': 0, 'meaning': 'MODE 3/A code as derived from the reply of the transponder',
                      'desc': ''},
                'V': {'val': 0, 'meaning': 'Code validated', 'desc': ''},
                'Mode3A': {'val': '4276',
                           'desc': 'Mode 3/A reply under the form of 4 digits in octal representation'},
                'G': {'val': 0, 'meaning': 'Default', 'desc': ''},
                'spare': {'val': 0, 'desc': 'Spare bit set to zero', 'const': 0}})

            self.assertEqual(packet[0]['I340']['POS'], {
                'RHO': {'max': 256.0, 'val': 186.6875, 'desc': 'Measured distance'},
                'THETA': {'val': 259.453125, 'desc': 'Measured azimuth'}})

            self.assertEqual(packet[0]['I105'], {
                'Lat': {'val': 44.73441302776337,
                        'desc': 'Latitude in WGS.84 in two\'s complement. Range -90 < latitude < 90 deg.'},
                                                                                                       'Lon': {
                'val': 13.0415278673172,
                'desc': 'Longitude in WGS.84 in two\'s complement. Range -180 < longitude < 180 deg.'}})
            self.assertEqual(packet[0]['I040'], {'TrkN': {'val': 4980, 'desc': 'Track number'}})
            self.assertEqual(packet[0]['I210'],
                                 {'Ax': {'val': 0.0, 'desc': 'Ax'}, 'Ay': {'val': 0.0, 'desc': 'Ay'}})
            self.assertEqual(packet[1]['I220'], {'RoC': {'val': 0.0, 'desc': 'Rate of Climb/Descent'}})
            self.assertEqual(packet[1]['I015'], {'SID': {'val': 4, 'desc': 'Service Identification'}})

            self.assertEqual(packet[1]['I290']['MDS'], {
                'MDS': {'val': 0.0, 'desc': 'Age of the last Mode S detection used to update the track'}})
            self.assertEqual(packet[1]['I290']['SSR'], {
                'SSR': {'val': 0.0, 'desc': 'Age of the last secondary detection used to update the track'}})

            self.assertEqual(packet[1]['I135'], {
                'QNH': {'meaning': 'No QNH correction applied', 'val': 0, 'desc': 'QNH'},
                'CTBA': {'max': 150000.0, 'min': -1500.0, 'val': 35000.0,
                         'desc': 'Calculated Track Barometric Alt'}})
            self.assertEqual(packet[1]['I136'], {
                 'MFL': {'max': 150000.0, 'min': -1500.0, 'val': 35000.0,
                        'desc': 'Measured Flight Level'}})
            self.assertEqual(packet[1]['I185'], {'Vx': {'max': 8191.75, 'min': -8192.0, 'val': 141.5, 'desc': 'Vx'},
                                                 'Vy': {'max': 8191.75, 'min': -8192.0, 'val': -170.75,
                                                        'desc': 'Vy'}})
            self.assertEqual(packet[1]['I080'], {'STP': {'meaning': 'default value', 'val': 0, 'desc': ''},
                                                     'MD5': {'meaning': 'No Mode 5 interrogation', 'val': 0,
                                                             'desc': ''},
                                                     'FPC': {'meaning': 'Flight plan correlated', 'val': 1,
                                                             'desc': ''},
                                                     'AMA': {
                                                         'meaning': 'track not resulting from amalgamation process',
                                                         'val': 0, 'desc': ''},
                                                     'CNF': {'meaning': 'Confirmed track', 'val': 0, 'desc': ''},
                                                     'TSE': {'meaning': 'default value', 'val': 0, 'desc': ''},
                                                     'ME': {'meaning': 'default value', 'val': 0, 'desc': ''},
                                                     'FX': {'meaning': 'End of data item', 'val': 0, 'desc': ''},
                                                     'CST': {'meaning': 'Default value', 'val': 0, 'desc': ''},
                                                     'PSR': {'meaning': 'Default value', 'val': 0, 'desc': ''},
                                                     'MDS': {'meaning': 'Default value', 'val': 0, 'desc': ''},
                                                     'MI': {'meaning': 'default value', 'val': 0, 'desc': ''},
                                                     'SRC': {'meaning': 'triangulation', 'val': 3,
                                                             'desc': 'Source of calculated track altitude for I062/130'},
                                                     'SIM': {'meaning': 'Actual track', 'val': 0, 'desc': ''},
                                                     'KOS': {'meaning': 'Background service used', 'val': 1,
                                                             'desc': ''},
                                                     'AFF': {'meaning': 'default value', 'val': 0, 'desc': ''},
                                                     'MRH': {
                                                         'meaning': 'Barometric altitude (Mode C) more reliable',
                                                         'val': 0, 'desc': 'Most Reliable Height'},
                                                     'MON': {'meaning': 'Multisensor track', 'val': 0, 'desc': ''},
                                                     'TSB': {'meaning': 'default value', 'val': 0, 'desc': ''},
                                                     'SUC': {'meaning': 'Default value', 'val': 0, 'desc': ''},
                                                     'MD4': {'meaning': 'No Mode 4 interrogation', 'val': 0,
                                                             'desc': ''},
                                                     'SPI': {'meaning': 'default value', 'val': 0, 'desc': ''},
                                                     'ADS': {
                                                         'meaning': 'Age of the last received ADS-B track update is higher than system dependent threshold',
                                                         'val': 1, 'desc': ''},
                                                     'AAC': {'meaning': 'Default value', 'val': 0, 'desc': ''},
                                                     'SSR': {'meaning': 'Default value', 'val': 0, 'desc': ''}})
            self.assertEqual(packet[1]['I070'], {'ToT': {'val': 30911.828125, 'desc': 'Time Of Track Information'}})
            self.assertEqual(packet[1]['I100'], {'Y': {'val': -36106.5, 'desc': 'Y'},
                                                     'X': {'val': -72564.5, 'desc': 'X'}})
            self.assertEqual(packet[1]['I200'], {'VERTA': {'meaning': 'Level', 'val': 0, 'desc': 'Vertical Rate'},
                                                 'spare': {'const': 0, 'val': 0, 'desc': 'Spare bit set to zero'},
                                                 'LONGA': {'meaning': 'Constant Groundspeed', 'val': 0,
                                                           'desc': 'Longitudinal Acceleration'},
                                                 'TRANSA': {'meaning': 'Constant Course', 'val': 0,
                                                            'desc': 'Transversal Acceleration'},
                                                 'ADF': {'meaning': 'No altitude discrepancy', 'val': 0,
                                                         'desc': 'Altitude Discrepancy Flag'}})
            self.assertEqual(packet[1]['I130'], {
                    'Alt': {'max': 150000.0, 'min': -1500.0, 'val': 35312.5, 'desc': 'Altitude'}})
            self.assertEqual(packet[1]['I060'],
                             {'CH': {'meaning': 'No Change', 'val': 0, 'desc': 'Change in Mode 3/A'},
                              'G': {'desc': 'Change in Mode 3/A', 'meaning': 'Default', 'val': 0},
                              'V': {'desc': 'Change in Mode 3/A', 'meaning': 'Code validated',
                                    'val': 0},
                              'spare': {'const': 0, 'val': 0, 'desc': 'Spare bits set to 0'},
                              'Mode3A': {'val': '2535',
                                         'desc': 'Mode-3/A reply in octal representation'}})

            self.assertEqual(packet[1]['I295']['MFL'], {'MFL': {'val': 0.0, 'desc': ''}})

            self.assertEqual(packet[1]['I390']['DEP'], {'DEP': {'desc': 'Departure Airport', 'val': 'EDDL'}})
            self.assertEqual(packet[1]['I390']['TAC'], {'TYPE': {'desc': 'Type of Aircraft', 'val': 'B738'}})
            self.assertEqual(packet[1]['I390']['DST'], {'DES': {'desc': 'Destination Airport', 'val': 'HELX'}})

            self.assertEqual(packet[1]['I390']['IFI'], {
                    'spare': {'const': 0, 'desc': 'spare bits set to zero', 'val': 0},
                    'NBR': {'desc': '', 'val': 29233709},
                    'TYP': {'meaning': 'Unit 1 internal flight number', 'desc': '', 'val': 1}})

            self.assertEqual(packet[1]['I390']['RDS'], {'NU1': {'desc': 'First number', 'val': ' '},
                                                        'LTR': {'desc': 'Letter', 'val': ' '},
                                                        'NU2': {'desc': 'Second number', 'val': ''}})

            self.assertEqual(packet[1]['I390']['WTC'],
                                 {'WTC': {'desc': 'Wake Turbulence Category', 'val': 'M'}})

            self.assertEqual(packet[1]['I390']['CSN'], {'CS': {'desc': 'Callsign', 'val': 'SXD4723'}})
            self.assertEqual(packet[1]['I390']['TAG'], {
                    'SIC': {'desc': 'System Identification Code', 'val': 100},
                    'SAC': {'desc': 'System Area Code', 'val': 25}})

            self.assertEqual(packet[1]['I390']['FCT'], {
                'spare': {'const': 0, 'desc': 'spare bit set to zero', 'val': 0},
                'FR1FR2': {'meaning': 'Instrument Flight Rules', 'desc': '', 'val': 0},
                'RVSM': {'meaning': 'Approved', 'desc': '', 'val': 1},
                'GATOAT': {'meaning': 'General Air Traffic', 'desc': '', 'val': 1},
                'HPR': {'meaning': 'Normal Priority Flight', 'desc': '', 'val': 0}})

            self.assertEqual(packet[1]['I390']['CFL'],
                                 {'CFL': {'desc': 'Current Cleared Flight Level', 'val': 350.0}})
            self.assertEqual(packet[1]['I010'], {'SAC': {'val': 25, 'desc': 'System Area Code'},
                                                 'SIC': {'val': 100, 'desc': 'System Identification Code'}})

            self.assertEqual(packet[1]['I340']['TYP'], {
                    'TYP': {'desc': 'Report Type', 'meaning': 'Single ModeS Roll-Call', 'val': 5},
                    'TST': {'desc': '', 'meaning': 'Real target report', 'val': 0},
                    'SIM': {'desc': '', 'meaning': 'Actual target report', 'val': 0},
                    'RAB': {'desc': '', 'meaning': 'Report from aircraft transponder', 'val': 0},
                    'spare': {'desc': 'Spare bits set to zero', 'val': 0, 'const': 0}})
            self.assertEqual(packet[1]['I340']['POS'], {
                'RHO': {'desc': 'Measured distance', 'val': 93.1953125, 'max': 256.0},
                'THETA': {'desc': 'Measured azimuth', 'val': 271.4666748046875}})
            self.assertEqual(packet[1]['I340']['MDA'], {
                    'G': {'desc': '', 'meaning': 'Default', 'val': 0},
                    'L': {'desc': '', 'meaning': 'MODE 3/A code as derived from the reply of the transponder',
                          'val': 0},
                    'V': {'desc': '', 'meaning': 'Code validated', 'val': 0},
                    'Mode3A': {'desc': 'Mode 3/A reply under the form of 4 digits in octal representation',
                               'val': '2535'},
                    'spare': {'desc': 'Spare bit set to zero', 'val': 0, 'const': 0}})
            self.assertEqual(packet[1]['I340']['MDC'], {
                'ModeC': {'min': -12.0, 'desc': 'Last Measured Mode C Code', 'val': 350.0, 'max': 1270.0},
                'CG': {'desc': '', 'meaning': 'Default', 'val': 0},
                'CV': {'desc': '', 'meaning': 'Code validated', 'val': 0}})
            self.assertEqual(packet[1]['I340']['SID'], {
                    'SIC': {'desc': 'System Identification Code', 'val': 13},
                    'SAC': {'desc': 'System Area Code', 'val': 25}})

            self.assertEqual(packet[1]['I380']['COM'], {
                'COM': {'val': 1, 'meaning': 'Comm. A and Comm. B capability',
                        'desc': 'Communications capability of the transponder'},
                'SSC': {'val': 1, 'meaning': 'Yes', 'desc': 'Specific service capability'},
                'STAT': {'val': 0, 'meaning': 'No alert, no SPI, aircraft airborne', 'desc': 'Flight Status'},
                'spare': {'val': 0, 'const': 0, 'desc': 'Spare bits set to zero'},
                'B1A': {'val': 1, 'desc': 'BDS 1,0 bit 16'},
                'B1B': {'val': 6, 'desc': 'BDS 1,0 bits 37/40'},
                'ARC': {'val': 1, 'meaning': '25 ft resolution', 'desc': 'Altitude reporting capability'},
                'AIC': {'val': 1, 'meaning': 'Yes', 'desc': 'Aircraft identification capability'}})
            self.assertEqual(packet[1]['I380']['ADR'], {'ADR': {'val': '3C0A55', 'desc': 'Target Address'}})
            self.assertEqual(packet[1]['I380']['ID'],
                             {'ACID': {'val': 'SXD4723 ', 'desc': 'Target Identification'}})

            self.assertEqual(packet[1]['I105'], {'Lat': {'val': 45.40080785751343,
                                                             'desc': 'Latitude in WGS.84 in two\'s complement. Range -90 < latitude < 90 deg.'},
                                                                                                         'Lon': {
            'val': 15.13318419456482,
            'desc': 'Longitude in WGS.84 in two\'s complement. Range -180 < longitude < 180 deg.'}})
            self.assertEqual(packet[1]['I040'], {'TrkN': {'val': 7977, 'desc': 'Track number'}})
            self.assertEqual(packet[1]['I210'],
                                 {'Ax': {'val': 0.0, 'desc': 'Ax'}, 'Ay': {'val': 0.0, 'desc': 'Ay'}})
            self.assertEqual(packet[2]['I015'], {'SID': {'val': 4, 'desc': 'Service Identification'}})
            self.assertEqual(packet[2]['I020'], {'BTN': {'val': 24, 'desc': 'Batch Number'}})
            self.assertEqual(packet[2]['I010'], {'SAC': {'val': 25, 'desc': 'Source Area Code'},
                                                 'SIC': {'val': 100, 'desc': 'Source Identification Code'}})
            self.assertEqual(packet[2]['I030'], {'ToD': {'val': 30913.0546875, 'desc': 'Time Of Message'}})
            self.assertEqual(packet[2]['I000'],
                             {'Typ': {'meaning': 'End of Batch', 'val': 2, 'desc': 'Message Type'}})


def main():
    unittest.main()


if __name__ == '__main__':
    main()



try:
    from setuptools import setup, Extension
except ImportError:
    from distutils.core import setup, Extension
import distutils.sysconfig
import shutil
import os.path
from os import listdir
import sys
import platform

exec(open('asterix/version.py').read())

CLASSIFIERS = [
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: GNU General Public License (GPL)',
    'Programming Language :: C',
    'Programming Language :: C++',
    'Programming Language :: Python :: 2.6',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3.2',
    'Programming Language :: Python :: 3.3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8'
]

try:
    shutil.rmtree("./build")
except(OSError):
    pass

# Remove the "-Wstrict-prototypes" compiler option, which isn't valid for C++.
cfg_vars = distutils.sysconfig.get_config_vars()
for key, value in cfg_vars.items():
    if type(value) == str:
        cfg_vars[key] = value.replace("-Wstrict-prototypes", "")

# If this is MacOSX set correct deployment target (otherwise compile may fail)
if sys.platform == 'darwin':
    os.environ['MACOSX_DEPLOYMENT_TARGET'] = platform.mac_ver()[0]

asterix_module = Extension('_asterix',
                           sources=['./src/python/asterix.c',
                                    './src/python/python_wrapper.c',
                                    './src/python/python_parser.cpp',
                                    './src/asterix/AsterixDefinition.cpp',
                                    './src/asterix/AsterixData.cpp',
                                    './src/asterix/Category.cpp',
                                    './src/asterix/DataBlock.cpp',
                                    './src/asterix/DataRecord.cpp',
                                    './src/asterix/DataItem.cpp',
                                    './src/asterix/DataItemBits.cpp',
                                    './src/asterix/DataItemDescription.cpp',
                                    './src/asterix/DataItemFormat.cpp',
                                    './src/asterix/DataItemFormatCompound.cpp',
                                    './src/asterix/DataItemFormatExplicit.cpp',
                                    './src/asterix/DataItemFormatFixed.cpp',
                                    './src/asterix/DataItemFormatRepetitive.cpp',
                                    './src/asterix/DataItemFormatVariable.cpp',
                                    './src/asterix/DataItemFormatBDS.cpp',
                                    './src/asterix/InputParser.cpp',
                                    './src/asterix/Tracer.cpp',
                                    './src/asterix/UAP.cpp',
                                    './src/asterix/UAPItem.cpp',
                                    './src/asterix/Utils.cpp',
                                    './src/asterix/XMLParser.cpp',
                                    ],

                           include_dirs=['./asterix/python', './src/asterix', './src/engine'],
                           extra_compile_args=['-DPYTHON_WRAPPER'],
                           extra_link_args=['-lexpat'])

f = open('README.rst')
try:
    README = f.read()
finally:
    f.close()

config_files = [os.path.join('./asterix/config/', f) for f in listdir('./asterix/config/') if
                os.path.isfile(os.path.join('./asterix/config/', f))]

sample_files = ['./asterix/sample_data/cat048.raw',
                './asterix/sample_data/cat062cat065.raw',
                './asterix/sample_data/cat_034_048.pcap',
                './asterix/sample_data/cat_062_065.pcap']

setup(name='asterix_decoder',
      packages=['asterix'],
      version=__version__,
      description="ASTERIX decoder in Python",
      keywords="asterix, eurocontrol, radar, track, croatiacontrol",
      long_description=README,
      ext_modules=[asterix_module],
      # data_files = [('asterix/config', config_files), ('asterix/sample_data', sample_files)],
      include_package_data=True,
      package_data={'asterix': config_files + sample_files},
      zip_safe=False,
      #       eager_resources = eager_files,
      author="Damir Salantic",
      author_email="damir.salantic@crocontrol.hr",
      download_url="https://github.com/CroatiaControlLtd/asterix",
      license="GPL",
      platforms=['any'],
      url="https://github.com/CroatiaControlLtd/asterix",
      classifiers=CLASSIFIERS,
      )



# !/usr/bin/python

import asterix
import unittest
import os
import sys


class AsterixInitTest(unittest.TestCase):

    def assertRaisesRegexpression(self, exc, text):
        if sys.version_info < (3, 0, 0):
            return self.assertRaisesRegexp(exc, text)
        else:
            return self.assertRaisesRegex(exc, text)

    def test_init_ok(self):
        sample_init = os.path.join(os.path.dirname(__file__), '../config/asterix_cat062_1_18.xml')
        ret = asterix.init(sample_init)
        self.assertIs(ret, 0)

    def test_init_type_error(self):
        with self.assertRaises(TypeError):
            asterix.init()

    def test_init_value_error(self):
        with self.assertRaises(ValueError):
            asterix.init(0)

    def test_init_io_error(self):
        with self.assertRaises(IOError):
            asterix.init("unknown_file.xml")

    def test_init_syntax_error(self):

        with self.assertRaisesRegexpression(SyntaxError, "Unknown tag: noCategory*"):
            sample_init = os.path.join(os.path.dirname(__file__), './parsing_error_1.xml')
            asterix.init(sample_init)

        with self.assertRaisesRegexpression(SyntaxError, "XMLParser : <Bits> without <Format>*"):
            sample_init = os.path.join(os.path.dirname(__file__), './parsing_error_2.xml')
            asterix.init(sample_init)


def main():
    unittest.main()


if __name__ == '__main__':
    main()
